@@ -37,13 +37,17 @@ class Hypervisor(object):
 
 		if not self.__info["hypervisor"]:
 			# On VMWare systems, the bios vendor string contains "VMWare".
-			if "VMWare" in self.system.bios_vendor:
+			if self.__is_hypervisor_vmware():
 				return "VMWare"
 
 			# VirtualBox got "innotek GmbH" as bios vendor.
-			elif self.system.bios_vendor == "innotek GmbH":
+			elif self.__is_hypervisor_virtualbox():
 				return "VirtualBox"
 
+			# Check for qemu.
+			elif self.__is_hypervisor_qemu():
+				return "Qemu"
+
 		return "unknown"
 
 	@property
@@ -65,7 +69,31 @@ class Hypervisor(object):
 		"""
 		return _fireinfo.is_virtualized() or \
 			"hypervisor" in self.system.cpu.flags or \
-			self.system.bios_vendor == "innotek GmbH"
+			self.__is_hypervisor_virtualbox() or \
+			self.__is_hypervisor_vmware() or \
+			self.__is_hypervisor_qemu()
+
+	def __is_hypervisor_virtualbox(self):
+		"""
+			Check for virtualbox hypervisor by comparing the bios vendor string
+			to "innotek GmbH".
+		"""
+		return self.system.bios_vendor == "innotek GmbH"
+
+	def __is_hypervisor_vmware(self):
+		"""
+			Check for the VMWare hypervisor by the VMWare Hypervisor port check.
+
+			http://kb.vmware.com/selfservice/microsites/search.do?language=en_US&cmd=displayKC&externalId=1009458
+		"""
+		return self.system.bios_vendor.startswith("VMWare-") and \
+			_fireinfo.vmware_hypervisor_port_check()
+
+	def __is_hypervisor_qemu(self):
+		"""
+			Check for old qemu emulator.
+		"""
+		return self.system.bios_vendor == "Bochs"
 
 
 if __name__ == "__main__":
